import React, { Component } from 'react'
import { DndProvider } from 'react-dnd'
import { withRouter } from 'react-router'
import { is, fromJS } from 'immutable'
import HTML5Backend from 'react-dnd-html5-backend'
import { ConfigProvider, notification, Modal, Collapse, Switch, Button, message, Spin, Typography } from 'antd'
import { DoubleLeftOutlined, DoubleRightOutlined, HomeOutlined, LoginOutlined, ArrowLeftOutlined, EyeOutlined, EyeInvisibleOutlined } from '@ant-design/icons'
import moment from 'moment'
import md5 from 'md5'

import Api from '@/api'
import Utils, { setGLOBFuncs } from '@/utils/utils.js'
import { getTables, getFuncsAndInters } from '@/utils/utils-custom.js'
import antdZhCN from 'antd/es/locale/zh_CN'
import MKEmitter from '@/utils/events.js'
import MenuUtils from '@/utils/utils-custom.js'
import asyncComponent from '@/utils/asyncComponent'

import '@/assets/css/design.scss'
import './index.scss'

const { Panel } = Collapse
const { confirm } = Modal
const { Paragraph } = Typography

const MenuForm = asyncComponent(() => import('./menuform'))
const Debug = asyncComponent(() => import('@/menu/debug'))
const Header = asyncComponent(() => import('@/menu/header'))
const PopView = asyncComponent(() => import('@/views/menudesign/popview'))
const Transfer = asyncComponent(() => import('@/pc/transfer'))
const Versions = asyncComponent(() => import('@/menu/versions'))
const MenuShell = asyncComponent(() => import('@/pc/menushell'))
const ViewNodes = asyncComponent(() => import('@/menu/viewnodes'))
const SourceWrap = asyncComponent(() => import('@/pc/modulesource'))
const Modulecell = asyncComponent(() => import('@/menu/modulecell'))
const TableNodes = asyncComponent(() => import('@/menu/tablenodes'))
const CreateView = asyncComponent(() => import('@/pc/createview'))
const BgController = asyncComponent(() => import('@/pc/bgcontroller'))
const Quotecomponent = asyncComponent(() => import('@/pc/quotecomponent'))
const PasteController = asyncComponent(() => import('@/menu/pastecontroller'))
const StyleController = asyncComponent(() => import('@/menu/stylecontroller'))
const ReplaceField = asyncComponent(() => import('@/menu/replaceField'))
const NormalCss = asyncComponent(() => import('@/menu/normalCss'))
const SysInterface = asyncComponent(() => import('@/menu/sysinterface'))
const UrlFieldComponent = asyncComponent(() => import('@/menu/urlfieldcomponent'))
const PictureController = asyncComponent(() => import('@/menu/picturecontroller'))
const ModalController = asyncComponent(() => import('@/menu/modalconfig/controller'))
const StyleCombController = asyncComponent(() => import('@/menu/stylecombcontroller'))
const StyleCombControlButton = asyncComponent(() => import('@/menu/stylecombcontrolbutton'))
const TableComponent = asyncComponent(() => import('@/templates/sharecomponent/tablecomponent'))

sessionStorage.setItem('appType', 'pc')        // 应用类型
sessionStorage.setItem('typename', 'pc')
document.body.className = ''

class MenuDesign extends Component {
  state = {
    loading: true,
    MenuId: '',
    MenuName: '',
    MenuNo: '',
    activeKey: 'basedata',
    menuloading: false,
    oriConfig: null,
    config: null,
    settingshow: sessionStorage.getItem('settingshow') !== 'false',
    controlshow: sessionStorage.getItem('controlshow') !== 'false',
    comloading: false,
    eyeopen: false,
    view: '',
    popConfig: null,
    needUpdate: false
  }

  UNSAFE_componentWillMount() {
    if (sessionStorage.getItem('devError') === 'true') {
      sessionStorage.clear()
      window.history.replaceState(null, null, window.location.href.split('#')[0] + '#/login')
      window.location.reload()
      return
    }
    
    if (!sessionStorage.getItem('UserID')) {
      sessionStorage.removeItem('appType')
      this.props.history.replace('/login')
      return
    }
    
    if (window.GLOB.memberLevel < 30) return

    sessionStorage.setItem('editMenuType', 'menu') // 编辑菜单类型

    window.GLOB.curDate = moment().format('YYYY-MM-DD')
    window.GLOB.TabsMap = new Map()          // 缓存用户操作的标签页
    window.GLOB.CacheIndependent = new Map()
    window.GLOB.urlFields = []               // url变量
    window.GLOB.customMenu = null            // 保存菜单信息
    window.GLOB.developing = true

    try {
      let param = JSON.parse(window.decodeURIComponent(window.atob(this.props.match.params.param)))

      if (param.type === 'app') {
        sessionStorage.setItem('appId', param.ID || '')
        sessionStorage.setItem('appName', param.remark || '')
        sessionStorage.setItem('lang', param.lang || 'zh-CN')
        sessionStorage.setItem('kei_no', param.kei_no || '')
        sessionStorage.setItem('sysBgColor', param.sysBgColor || '#ffffff')

        this.getAppMessage(param.MenuID)
      } else if (param.type === 'view') {
        this.setState({
          MenuId: param.MenuID
        }, () => {
          this.getMenuParam(param)
        })
      }
    } catch (e) {
      notification.warning({
        top: 92,
        message: '菜单信息解析错误！',
        duration: 5
      })
    }
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (this.props.match.params.param !== nextProps.match.params.param) {
      window.location.reload()
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    if (!sessionStorage.getItem('UserID')) return
    if (window.GLOB.memberLevel < 30) {
      document.getElementById('mk-pc-design-view').innerHTML = '<div style="text-align: center; font-size: 30px; margin-top: 40vh; height: 100vh; background: #fff;">本应用没有PC端页面的编辑权限，请联系管理员！</div>'
      return
    }
    MKEmitter.addListener('changePopview', this.initPopview)
    MKEmitter.addListener('changeEditMenu', this.changeEditMenu)
    MKEmitter.addListener('triggerMenuSave', this.triggerMenuSave)

    setTimeout(() => {
      this.getRoleFields()
      setGLOBFuncs()
    }, 1000)

    document.onkeydown = (event) => {
      let e = event || window.event
      let keyCode = e.keyCode || e.which || e.charCode
      let preKey = ''

      if (e.ctrlKey) {
        preKey = 'ctrl'
      }
      if (e.shiftKey) {
        preKey = 'shift'
      } else if (e.altKey) {
        preKey = 'alt'
      }
      
      if (!preKey || !keyCode) return
      
      let _shortcut = `${preKey}+${keyCode}`

      if (_shortcut === 'ctrl+83') {
        let modals = document.querySelectorAll('.mk-pop-modal')
        let msg = null
        for (let i = 0; i < modals.length; i++) {
          if (msg) {
            break
          }

          let node = modals[i].querySelector('.mk-com-name')

          if (node) {
            msg = node.innerText
          }
        }
        if (msg) {
          notification.warning({
            top: 92,
            message: '请保存' + msg,
            duration: 5
          })
          return false
        }

        let node = document.getElementById('save-modal-config')
        if (!node) {
          node = document.getElementById('save-pop-config')
        }
        if (!node) {
          node = document.getElementById('save-config')
        }

        if (node) {
          node.click()
        }
        return false
      }
    }
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('changePopview', this.initPopview)
    MKEmitter.removeListener('changeEditMenu', this.changeEditMenu)
    MKEmitter.removeListener('triggerMenuSave', this.triggerMenuSave)
  }

  triggerMenuSave = () => {
    if (this.state.view === 'popview') return

    this.submitConfig()
  }

  changeEditMenu = (menu) => {
    const { oriConfig, config } = this.state

    if (!oriConfig || !is(fromJS(oriConfig), fromJS(config))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    }

    let param = {
      MenuID: menu.MenuID,
      copyMenuId: menu.copyMenuId || '',
      clearMenu: menu.clearMenu !== 'false',
      type: 'view'
    }

    // param.MenuNo = menu.MenuNo || ''
    param.MenuName = menu.MenuName || ''

    param = window.btoa(window.encodeURIComponent(JSON.stringify(param)))

    if (param === this.props.match.params.param) {
      window.location.reload()
      return
    }

    this.props.history.push('/pcdesign/' + param)
  }

  getAppMessage = (MenuID) => {
    Api.getCloudConfig({
      func: 's_get_keyids',
      bid: sessionStorage.getItem('appId')
    }).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return
      }

      let homeId = ''
      let appViewList = []
      if (res.data && res.data.length > 0) {
        appViewList = res.data
        appViewList.forEach(item => {
          if (item.keys_type === 'index') {
            homeId = item.keys_id
          }
        })
      }

      if (!homeId) {
        homeId = MenuID || Utils.getuuid()

        let param = {
          func: 's_kei_link_keyids_addupt',
          BID: sessionStorage.getItem('appId'),
          exec_type: 'x',
          LText: ''
        }

        appViewList.unshift({
          appkey: window.GLOB.appkey || '',
          bid: sessionStorage.getItem('appId') || '',
          kei_no: sessionStorage.getItem('kei_no') || '',
          keys_id: homeId,
          keys_type: 'index',
          remark: '首页'
        })

        param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
        param.LText = param.LText.join(' union all ')
        param.LText = Utils.formatOptions(param.LText, 'x')
  
        param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
        param.secretkey = Utils.encrypt('', param.timestamp)

        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
            this.props.history.replace('/pcdesign/' + window.btoa(window.encodeURIComponent(JSON.stringify({MenuID: homeId, type: 'view'}))))
          }
        })
      } else {
        sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
        this.props.history.replace('/pcdesign/' + window.btoa(window.encodeURIComponent(JSON.stringify({MenuID: MenuID || homeId, type: 'view'}))))
      }
    })
  }

  initPopview = (card, btn) => {
    const { config } = this.state

    if (!this.checkBase()) {
      return
    }

    let _btn = fromJS(btn).toJS()

    if (_btn.config) {
      _btn.config.uuid = _btn.uuid
      _btn.config.MenuID = _btn.uuid
      _btn.config.ParentId = card.uuid
      _btn.config.MenuName = _btn.label

      _btn.config.components = this.updateComponents(_btn.config.components || [])
    } else {
      _btn.config = {
        uuid: _btn.uuid,
        MenuID: _btn.uuid,
        ParentId: card.uuid,
        enabled: false,
        MenuName: _btn.label,
        tables: config.tables || [],
        Template: 'CustomPage',
        components: [],
        viewType: 'popview',
        style: { backgroundColor: '#ffffff', backgroundImage: '', paddingTop: '16px', paddingBottom: '40px', paddingLeft: '16px', paddingRight: '16px' }
      }
    }

    this.setState({view: 'popview', popConfig: _btn})
  }

  submitPopConfig = (btnconfig) => {
    let parents = {[btnconfig.ParentId]: true}
    let popbtns = {[btnconfig.uuid]: fromJS(btnconfig).toJS()}
    let config = fromJS(this.state.config).toJS()

    config.components = this.setPopView(config.components, parents, popbtns)

    this.setState({ config }, () => {
      this.submitConfig()
    })
  }

  setPopView = (components, parents, popbtns) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.setPopView(tab.components, parents, popbtns)
        })
      } else if (item.type === 'group') {
        item.components = this.setPopView(item.components, parents, popbtns)
      } else if (parents[item.uuid]) {
        this.setpopConfig(item, popbtns)
      }
      return item
    })
  }

  setpopConfig = (config, popbtns) => {
    config.subcards && config.subcards.forEach(item => {
      item.elements.forEach(cell => {
        if (cell.eleType !== 'button') return
        if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
          cell.config = popbtns[cell.uuid]
        }
      })
    })

    if (config.cols) {
      let loopCol = (cols) => {
        cols.forEach(col => {
          if (col.type === 'colspan') {
            loopCol(col.subcols)
          } else if (col.type === 'custom') {
            col.elements.forEach(cell => {
              if (cell.eleType !== 'button') return
              if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
                cell.config = popbtns[cell.uuid]
              }
            })
          }
        })
      }
      loopCol(config.cols)
    }

    config.elements && config.elements.forEach(cell => {
      if (cell.eleType !== 'button') return
      if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
        cell.config = popbtns[cell.uuid]
      }
    })
  
    config.action && config.action.forEach(cell => {
      if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
        cell.config = popbtns[cell.uuid]
      }
    })

    config.$tables = getTables(config)
  }

  closePop = () => {
    const {config} = this.state

    sessionStorage.setItem('editMenuType', 'menu')

    window.GLOB.urlFields = config.urlFields || []
    window.GLOB.customMenu = config

    this.setState({view: '', popConfig: null})
  }

  closeView = () => {
    const { oriConfig, config } = this.state

    if (!config) {
      window.close()
    } else if (!oriConfig || !is(fromJS(oriConfig), fromJS(config))) {
      confirm({
        title: '配置信息未保存，确定关闭吗？',
        content: '',
        onOk() {
          window.close()
        },
        onCancel() {}
      })
    } else {
      window.close()
    }
  }

  backView = () => {
    const { oriConfig, config } = this.state

    if (!config) {
      window.history.back()
    } else if (!oriConfig || !is(fromJS(oriConfig), fromJS(config))) {
      confirm({
        title: '配置信息未保存，确定后退吗？',
        content: '',
        onOk() {
          window.history.back()
        },
        onCancel() {}
      })
    } else {
      window.history.back()
    }
  }

  getMenuParam = (urlParam) => {
    const { MenuId } = this.state

    let param = {
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: 'pc',
      MenuID: MenuId
    }

    Api.getCloudConfig(param).then(result => {
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        this.setState({loading: false})
        return
      } else if (!result.LongParam && urlParam.copyMenuId) {
        this.getCopyParam(urlParam)
      } else {
        let config = null

        try {
          config = result.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(result.LongParam))) : null
        } catch (e) {
          console.warn('Parse Failure')
          config = null
        }

        if (!config) {
          config = {
            version: 2.0,
            uuid: MenuId,
            MenuID: MenuId,
            Template: 'webPage',
            enabled: false,
            MenuName: urlParam.MenuName || '',
            MenuNo: '',
            tables: [],
            components: [],
            viewType: 'menu',
            style: {
              backgroundColor: sessionStorage.getItem('sysBgColor') || '#ffffff', backgroundImage: '', paddingLeft: '20px', paddingRight: '20px'
            }
          }
        }

        config.uuid = MenuId
        config.MenuID = MenuId
        config.Template = 'webPage'
        config.open_edition = result.open_edition || ''
        window.GLOB.urlFields = config.urlFields || []

        if (config.version !== 2.0) {
          config.components = this.collectTB(config.components)
          config.version = 2.0

          this.setState({
            needUpdate: true
          })
        } else {
          config.components = this.updateComponents(config.components)
        }

        let navItem = null
        config.components.forEach(item => {
          if (item.type === 'navbar') {
            navItem = fromJS(item).toJS()
          }
        })

        if (!navItem) {
          this.setState({
            oriConfig: fromJS(config).toJS(),
            config: config,
            loading: false
          })

          window.GLOB.customMenu = config
        } else {
          this.jointComponents(config, navItem)
        }
      }
    })
    this.getAppMenus()
  }

  updateComponents = (components) => { // 兼容性升级 table
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.updateComponents(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.updateComponents(item.components)
      } else if (item.type === 'table') {
        item.cols = item.cols.map(col => {
          if (col.type === 'action') {
            col.type = 'custom'
          }
          return col
        })
      }

      return item
    })
  }

  collectTB = (components) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          delete tab.floor
          delete tab.hasSearch
          delete tab.parentId

          tab.components = this.collectTB(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.collectTB(item.components)
      } else if (!['search', 'navbar', 'login', 'topbar', 'officialAccount'].includes(item.type)) {
        item.$tables = getTables(item)
      }

      if (item.subtype === 'tablecard') { // 兼容
        item.type = 'card'
      } else if (item.type === 'table') {
        item.cols = item.cols.map(col => {
          if (col.type === 'action') {
            col.type = 'custom'
          }
          return col
        })
      }

      delete item.tabId
      delete item.parentId
      delete item.btnlog
      delete item.floor
      delete item.dataName

      return item
    })
  }

  getAppMenus = () => {
    let _param = {
      func: 's_get_app_menus',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: 'pc',
      LText: `select '${window.GLOB.appkey}'`,
      timestamp: moment().format('YYYY-MM-DD HH:mm:ss')
    }

    _param.secretkey = Utils.encrypt(_param.LText, _param.timestamp)

    Api.getCloudConfig(_param).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return
      }

      let appIndeList = sessionStorage.getItem('appViewList')
      appIndeList = JSON.parse(appIndeList)
      appIndeList = appIndeList.map(item => (item.keys_type === 'navbar' ? item.keys_id : '')).join(',')

      let menus = res.menus.filter(item => appIndeList.indexOf(item.MenuID) === -1)
      menus = menus.map(item => {
        item.value = item.MenuID
        if (item.MenuNo) {
          item.label = item.MenuName + '（' + item.MenuNo + '）'
        } else {
          item.label = item.MenuName
        }
        return item
      })
      sessionStorage.setItem('appMenus', JSON.stringify(menus))
      sessionStorage.setItem('allMenus', JSON.stringify(res.menus || []))
    })
  }

  getCopyParam = (urlParam) => {
    const { MenuId } = this.state

    let param = {
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: 'pc',
      MenuID: urlParam.copyMenuId
    }

    Api.getCloudConfig(param).then(result => {
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        this.setState({loading: false})
        return
      }

      let config = null

      try {
        config = result.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(result.LongParam))) : null
      } catch (e) {
        console.warn('Parse Failure')
        config = null
      }
      
      if (!config) {
        config = {
          version: 2.0,
          uuid: MenuId,
          MenuID: MenuId,
          Template: 'webPage',
          enabled: false,
          MenuName: '',
          MenuNo: '',
          tables: [],
          components: [],
          viewType: 'menu',
          style: {
            backgroundColor: sessionStorage.getItem('sysBgColor') || '#ffffff', backgroundImage: '', paddingLeft: '20px', paddingRight: '20px'
          }
        }
      } else {
        let commonId = Utils.getuuid()
        if (config.interfaces && config.interfaces.length > 0) {
          config.interfaces = config.interfaces.map(inter => {
            inter.uuid = md5(commonId + inter.uuid)
            return inter
          })
        }
        config.components = MenuUtils.resetConfig(config.components, commonId, urlParam.clearMenu)

        if (config.version !== 2.0) {
          config.components = this.collectTB(config.components)
          config.version = 2.0

          this.setState({
            needUpdate: true
          })
        }

        config.enabled = false
        message.success('复制成功，保存后生效。')
      }
      
      config.uuid = MenuId
      config.MenuID = MenuId
      config.Template = 'webPage'
      config.open_edition = ''
      config.MenuName = urlParam.MenuName || ''
      config.MenuNo = ''

      let navItem = null
      config.components.forEach(item => {
        if (item.type === 'navbar') {
          navItem = fromJS(item).toJS()
        }
      })

      if (!navItem) {
        this.setState({
          oriConfig: fromJS(config).toJS(),
          config: config,
          loading: false
        })
        window.GLOB.customMenu = config
      } else {
        this.jointComponents(config, navItem)
      }
    })
  }

  jointComponents = (config, navItem) => {
    Api.getCloudConfig({
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: 'pc',
      MenuID: navItem.uuid
    }).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })

        config.components = config.components.filter(item => item.type !== 'navbar')
      } else {
        let _config = null
        try {
          _config = res.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(res.LongParam))) : null
        } catch (e) {
          console.warn('Parse Failure')
          _config = null
        }

        if (_config) {
          _config.open_edition = res.open_edition || ''
          config.components = config.components.map(item => {
            if (item.type === 'navbar') {
              return _config
            }
            return item
          })

          window.GLOB.CacheIndependent.set(navItem.uuid, fromJS(_config).toJS())
        } else {
          config.components = config.components.filter(item => item.type !== 'navbar')
        }
      }

      this.setState({
        oriConfig: fromJS(config).toJS(),
        config: config,
        loading: false
      })

      window.GLOB.customMenu = config
    })
  }

  getMenuMessage = (tbs) => {
    const { config } = this.state
    let nodes = {type: 'view', version: '1.0', key: config.uuid, title: config.MenuName, children: []}

    let traversal = (components) => {
      let list = []
      components.forEach(item => {
        if (item.$tables) {
          tbs.push(...item.$tables)
        }

        if (item.plot && item.plot.permission !== 'true') {
          return
        } else if (item.type === 'login' || item.type === 'navbar') {
          return
        } else if (!['tabs', 'group'].includes(item.type) && item.wrap && item.wrap.permission !== 'true') {
          return
        }

        let m = {
          key: item.uuid,
          title: item.name,
          children: []
        }

        if (item.type === 'tabs') {
          let tabs = []
          let mm = []
          item.subtabs.forEach(tab => {
            let s = traversal(tab.components)

            if (s.length === 0) return

            tabs.push({
              key: tab.uuid,
              title: tab.label,
              children: s
            })

            mm.push(...s)
          })

          if (tabs.length > 0) {
            m.children = tabs
          }

          if (item.setting.permission === 'true') {
            list.push(m)
          } else if (mm.length) {
            list.push(...mm)
          }
          
          return
        } else if (item.type === 'group') {
          m.children = traversal(item.components)

          if (item.setting.permission === 'true') {
            list.push(m)
          } else if (m.children.length) {
            list.push(...m.children)
          }
          
          return
        } else if (item.type === 'card' || item.type === 'carousel' || item.type === 'timeline') {
          item.action && item.action.forEach(btn => {
            if (btn.hidden === 'true' || btn.permission === 'false') return

            m.children.push({
              key: btn.uuid,
              title: btn.label,
            })
          })
          item.subcards.forEach(card => {
            card.elements && card.elements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

              m.children.push({
                key: cell.uuid,
                title: cell.label,
              })
            })

            card.backElements && card.backElements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

              m.children.push({
                key: cell.uuid,
                title: cell.label,
              })
            })
          })
        } else if (item.type === 'balcony') {
          item.elements && item.elements.forEach(cell => {
            if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

            m.children.push({
              key: cell.uuid,
              title: cell.label,
            })
          })
        } else if (item.type === 'form') {
          if (item.subtype !== 'simpleform') {
            m.children = item.subcards.map(m => {
              return {
                key: m.uuid,
                title: m.setting.title
              }
            })
          }
        } else if (item.type === 'table') {
          item.action.forEach(btn => {
            if (btn.hidden === 'true' || btn.permission === 'false') return

            m.children.push({
              key: btn.uuid,
              title: btn.label,
            })
          })
          let loopCol = (cols) => {
            cols.forEach(col => {
              if (col.type === 'colspan') {
                loopCol(col.subcols)
              } else if (col.type === 'custom') {
                col.elements.forEach(cell => {
                  if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return
                  m.children.push({
                    key: cell.uuid,
                    title: cell.label,
                  })
                })
              }
            })
          }
          loopCol(item.cols)
        }

        list.push(m)
      })

      return list
    }

    let trees = traversal(config.components)

    nodes.children = trees

    if (config.components.findIndex(item => item.type === 'login') > -1) {
      nodes.login = true
      nodes.children = []
    }
    if (nodes.children.length === 0) {
      nodes.pass = true
    }

    if (config.interfaces) {
      config.interfaces.forEach(item => {
        if (item.$tables) {
          tbs.push(...item.$tables)
        }
      })
    }

    return nodes
  }

  getSubMenus = () => {
    const { config } = this.state
    let menus = []
    let menuObj = {}
    let allMenus = JSON.parse(sessionStorage.getItem('allMenus'))

    allMenus.forEach(item => {
      menuObj[item.MenuID] = item
    })

    let traversal = (components) => {
      components.forEach(item => {
        if (item.type === 'login') {
          if (item.wrap.linkmenu && menuObj[item.wrap.linkmenu]) {
            menus.push(menuObj[item.wrap.linkmenu])
          }
        } else if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            traversal(tab.components)
          })
        } else if (item.type === 'group') {
          traversal(item.components)
        } else if (item.type === 'card' || item.type === 'carousel' || item.type === 'timeline') {
          item.action && item.action.forEach(btn => {
            if (btn.linkmenu && menuObj[btn.linkmenu]) {
              menus.push(menuObj[btn.linkmenu])
            } else if (btn.openmenu && menuObj[btn.openmenu]) {
              menus.push(menuObj[btn.openmenu])
            }
          })
          item.subcards.forEach(card => {
            if (card.setting.click === 'menu' && menuObj[card.setting.menu]) {
              menus.push(menuObj[card.setting.menu])
            } else if (card.setting.click === 'menus' && card.menus) {
              card.menus.forEach(m => {
                if (menuObj[m.menu]) {
                  menus.push(menuObj[m.menu])
                }
              })
            }
            card.elements && card.elements.forEach(cell => {
              if (cell.linkmenu && menuObj[cell.linkmenu]) {
                menus.push(menuObj[cell.linkmenu])
              } else if (cell.openmenu && menuObj[cell.openmenu]) {
                menus.push(menuObj[cell.openmenu])
              }
            })

            card.backElements && card.backElements.forEach(cell => {
              if (cell.linkmenu && menuObj[cell.linkmenu]) {
                menus.push(menuObj[cell.linkmenu])
              } else if (cell.openmenu && menuObj[cell.openmenu]) {
                menus.push(menuObj[cell.openmenu])
              }
            })
          })
        } else if (item.type === 'balcony') {
          item.elements && item.elements.forEach(cell => {
            if (cell.linkmenu && menuObj[cell.linkmenu]) {
                menus.push(menuObj[cell.linkmenu])
              } else if (cell.openmenu && menuObj[cell.openmenu]) {
                menus.push(menuObj[cell.openmenu])
              }
          })
        } else if (item.type === 'form') {
          item.subcards.forEach(m => {
            if (m.subButton && m.subButton.linkmenu && menuObj[m.subButton.linkmenu]) {
              menus.push(menuObj[m.subButton.linkmenu])
            }
          })
        } else if (item.type === 'table') {
          item.action && item.action.forEach(btn => {
            if (btn.linkmenu && menuObj[btn.linkmenu]) {
              menus.push(menuObj[btn.linkmenu])
            } else if (btn.openmenu && menuObj[btn.openmenu]) {
              menus.push(menuObj[btn.openmenu])
            }
          })
          let loopCol = (cols) => {
            cols.forEach(col => {
              if (col.type === 'colspan') {
                loopCol(col.subcols)
              } else if (col.type === 'custom') {
                col.elements.forEach(cell => {
                  if (cell.linkmenu && menuObj[cell.linkmenu]) {
                    menus.push(menuObj[cell.linkmenu])
                  } else if (cell.openmenu && menuObj[cell.openmenu]) {
                    menus.push(menuObj[cell.openmenu])
                  }
                })
              }
            })
          }
          loopCol(item.cols)
        }
      })
    }

    traversal(config.components)

    let map = new Map()
    menus = menus.filter(m => {
      if (map.has(m.MenuID)) return false
      map.set(m.MenuID, true)
      return true
    })

    return menus
  }

  checkBase = () => {
    const { config } = this.state

    if (!config.MenuName || !config.MenuNo || (config.cacheUseful === 'true' && !config.cacheTime)) {
      notification.warning({
        top: 92,
        message: '请完善菜单基本信息！',
        duration: 5
      })
      sessionStorage.setItem('settingshow', 'true')
      this.setState({
        settingshow: true,
        activeKey: 'basedata'
      })
      return false
    }
    return true
  }

  resetSyncQuery = (components) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.resetSyncQuery(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.resetSyncQuery(item.components)
      } else if (item.setting && item.setting.interType === 'system' && item.setting.sync === 'true') {
        let sql = ''
        if (item.setting.execute !== 'false' && item.setting.dataresource) {
          sql = item.setting.dataresource
        }

        item.scripts && item.scripts.forEach(script => {
          if (script.status === 'false') return

          sql += script.sql
        })
       
        if (sql.length > 8000) {
          item.setting.sync = 'false'
        }
      }

      return item
    })
  }

  submitConfig = () => {
    let config = fromJS(this.state.config).toJS()

    if (!this.checkBase()) {
      return
    }

    this.setState({
      menuloading: true
    })

    setTimeout(() => {
      let _pass = this.verifyConfig(config)

      if (config.enabled && !_pass) {
        config.enabled = false
        config.force = true
      } else if (!config.enabled && config.force && _pass) {
        config.enabled = true
        delete config.force
      }

      if (config.cacheUseful !== 'true') {
        config.components = this.resetSyncQuery(config.components)
      }

      let tbs = []
      let roleParam = this.getMenuMessage(tbs)

      let subMenus = this.getSubMenus()
      let menus_used_list = subMenus.map(m => `'${config.uuid}','${config.MenuName || ''}','${config.MenuNo || ''}','${m.MenuID}','${m.MenuName}'`).join(';')
      menus_used_list = window.btoa(window.encodeURIComponent(menus_used_list || 'del'))

      let arr = []
      tbs = tbs.filter(tb => {
        let _tb = tb.toLowerCase()

        if (arr.includes(_tb)) return false
        arr.push(_tb)

        return true
      })
      tbs.sort()
      if (tbs.length && sessionStorage.getItem('mk_tb_names')) {
        let names = sessionStorage.getItem('mk_tb_names')
        tbs = tbs.filter(tb => names.indexOf(',' + tb.toLowerCase() + ',') > -1)
      }
      tbs = tbs.map(tb => `'${tb}'`).join(';')

      let key = md5(config.uuid + sessionStorage.getItem('kei_no') + 'pc' + sessionStorage.getItem('lang') + tbs.toLowerCase())
      let url = ''

      if (config.tbkey === key) {
        key = ''
      } else {
        let urlparam = {
          type: 'app',
          MenuID: config.uuid,
          ID: sessionStorage.getItem('appId') || '',
          remark: sessionStorage.getItem('appName') || '',
          lang: sessionStorage.getItem('lang') || '',
          kei_no: sessionStorage.getItem('kei_no') || '',
          typename: 'pc',
          sysBgColor: sessionStorage.getItem('sysBgColor') || '',
          MenuName: config.MenuName || '',
        }

        url = window.btoa(window.encodeURIComponent(JSON.stringify(urlparam)))
        config.tbkey = key
      }

      let interfaces = getFuncsAndInters(config)
      roleParam.interfaces = interfaces

      let param = {
        func: 'sPC_TrdMenu_AddUpt',
        FstID: 'mk_app',
        SndID: 'mk_app',
        ParentID: 'mk_app',
        MenuID: config.uuid,
        MenuNo: config.MenuNo || '',
        EasyCode: '',
        Template: 'webPage',
        TypeCharOne: sessionStorage.getItem('kei_no'),
        Typename: 'pc',
        MenuName: config.MenuName || '',
        PageParam: JSON.stringify({Template: 'webPage', interfaces}),
        menus_rolelist: window.btoa(window.encodeURIComponent(JSON.stringify(roleParam))),
        open_edition: config.open_edition,
        LText: '',
        LTexttb: '',
        menus_used_list,
        debug_md5: key,
        debug_url: url,
        debug_list: window.btoa(tbs)
      }

      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt('', param.timestamp)

      let _config = fromJS(config).toJS()
      let NavBar = null
      _config.components = _config.components.map(item => {
        if (item.type === 'navbar') {
          NavBar = fromJS(item).toJS()
          return {
            type: 'navbar',
            uuid: item.uuid
          }
        }
        return item
      })

      param.LongParam = window.btoa(window.encodeURIComponent(JSON.stringify(_config)))

      new Promise(resolve => {
        if (!NavBar) {
          resolve({status: true})
        } else {
          let _item = window.GLOB.CacheIndependent.get(NavBar.uuid) || {}
          NavBar.open_edition = _item.open_edition || ''
          if (is(fromJS(_item), fromJS(NavBar))) {
            resolve({status: true})
          } else {
            let roles = {
              type: 'navbar',
              version: '1.0',
              key: NavBar.uuid,
              title: NavBar.name,
              children: []
            }

            if (NavBar.wrap.permission === 'true') {
              roles.children = NavBar.menus.map(fst => {
                if (fst.property === 'classify' && fst.sublist.length > 0) {
                  return {
                    key: fst.MenuID,
                    title: fst.name,
                    children: fst.sublist.map(scd => {
                      if (scd.property === 'classify' && scd.sublist.length > 0) {
                        return {
                          key: scd.MenuID,
                          title: scd.name,
                          children: scd.sublist.map(thd => {
                            return { key: thd.MenuID, title: thd.name }
                          })
                        }
                      } else {
                        return { key: scd.MenuID, title: scd.name }
                      }
                    })
                  }
                } else {
                  return { key: fst.MenuID, title: fst.name }
                }
              })
            } else {
              roles.pass = true
            }

            let _param = {
              func: 'sPC_TrdMenu_AddUpt',
              FstID: 'mk_app',
              SndID: 'mk_app',
              ParentID: 'mk_app',
              MenuID: NavBar.uuid,
              MenuNo: NavBar.wrap.MenuNo || Utils.getuuid(),
              EasyCode: '',
              Template: NavBar.type,
              TypeCharOne: sessionStorage.getItem('kei_no'),
              Typename: 'pc',
              MenuName: NavBar.name || '',
              PageParam: JSON.stringify({Template: NavBar.type}),
              menus_rolelist: window.btoa(window.encodeURIComponent(JSON.stringify(roles))),
              open_edition: NavBar.open_edition,
              LText: '',
              LTexttb: ''
            }

            _param.LongParam = window.btoa(window.encodeURIComponent(JSON.stringify(NavBar)))
            _param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            _param.secretkey = Utils.encrypt('', _param.timestamp)

            Api.getCloudConfig(_param).then(res => {
              if (res.status) {
                NavBar.open_edition = res.open_edition || ''
                window.GLOB.CacheIndependent.set(NavBar.uuid, fromJS(NavBar).toJS())

                let appViewList = sessionStorage.getItem('appViewList')
                let _appViewList = JSON.parse(appViewList)

                let index = _appViewList.findIndex(item => item.keys_id === NavBar.uuid)
                
                if (index === -1) {
                  _appViewList.unshift({
                    appkey: window.GLOB.appkey || '',
                    bid: sessionStorage.getItem('appId') || '',
                    kei_no: sessionStorage.getItem('kei_no') || '',
                    keys_id: NavBar.uuid,
                    keys_type: 'navbar',
                    remark: NavBar.name
                  })
                } else {
                  _appViewList = _appViewList.map(item => {
                    if (item.keys_id === NavBar.uuid && item.remark !== NavBar.name) {
                      item.remark = NavBar.name
                    }
                    return item
                  })
                }

                let viewList = JSON.stringify(_appViewList)

                if (appViewList !== viewList) {
                  let kparam = {
                    func: 's_kei_link_keyids_addupt',
                    BID: sessionStorage.getItem('appId'),
                    exec_type: 'x',
                    LText: ''
                  }
        
                  kparam.LText = _appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
                  kparam.LText = kparam.LText.join(' union all ')
                  kparam.LText = Utils.formatOptions(kparam.LText, 'x')
            
                  kparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
                  kparam.secretkey = Utils.encrypt('', kparam.timestamp)
        
                  Api.getCloudConfig(kparam).then(result => {
                    if (result.status) {
                      sessionStorage.setItem('appViewList', viewList)
                    }

                    resolve(result)
                  }, this.netError)
                } else {
                  resolve(res)
                }
              } else {
                resolve(res)
              }
            }, this.netError)
          }
        }
      }).then(res => { // 页面保存
        if (!res || !res.status) return res

        return Api.getCloudConfig(param)
      }).then(res => {
        this.setState({
          menuloading: false
        })

        if (!res) return

        if (res.status) {
          config.open_edition = res.open_edition || ''

          this.setState({
            config,
            oriConfig: fromJS(config).toJS(),
            needUpdate: false
          })

          localStorage.setItem('pcUpdate', new Date().getTime() + ',' + config.uuid)

          notification.success({
            top: 92,
            message: '保存成功',
            duration: 2
          })
        } else {
          Modal.warning({
            width: 400,
            title: res.message,
            okText: '知道了'
          })
        }
        MKEmitter.emit('completeSave')
      }, this.netError)
    }, 300 + (+sessionStorage.getItem('mkDelay')))
  }

  netError = (error) => {
    this.setState({
      menuloading: false
    })

    if (!error) {
      notification.warning({
        top: 92,
        message: '保存失败，请检查网络是否正常。',
        duration: 5
      })
    }
    MKEmitter.emit('completeSave')
  }

  getRoleFields = () => {
    if (sessionStorage.getItem('sysRoles') || sessionStorage.getItem('permFuncField')) return
    Api.getCloudConfig({func: 'sPC_Get_Roles_sModular'}).then(res => {
      if (res.status) {
        let _permFuncField = []
        let _sysRoles = []

        if (res.Roles && res.Roles.length > 0) {
          _sysRoles = res.Roles.map(role => {
            return {
              uuid: Utils.getuuid(),
              value: role.RoleID,
              text: role.RoleName
            }
          })
        }

        if (res.sModular && res.sModular.length > 0) {
          res.sModular.forEach(field => {
            if (field.ModularNo) {
              _permFuncField.push(field.ModularNo)
            }
          })
          _permFuncField = _permFuncField.sort()
        }

        sessionStorage.setItem('sysRoles', JSON.stringify(_sysRoles))
        sessionStorage.setItem('permFuncField', JSON.stringify(_permFuncField))
      }
    })
  }

  onEnabledChange = () => {
    const { config } = this.state

    let _config = {...config, enabled: !config.enabled}

    delete _config.force

    if (!_config.enabled) {
      this.setState({
        config: _config
      })
    } else if (this.verifyConfig(_config)) {
      this.setState({
        config: _config
      })
    }
  }

  verifyConfig = (config) => {
    let error = ''

    let check = (components) => {
      components.forEach(item => {
        if (error) return
        if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            check(tab.components)
          })
          return
        } else if (item.type === 'group') {
          check(item.components)
          return
        } else if (!item.errors || item.errors.length === 0) {
          return
        }
        
        item.errors.forEach(err => {
          if (err.level !== 0 || error) return
          error = `组件《${item.name}》${err.detail}`
        })
      })
    }

    check(config.components)

    if (config.enabled && error) {
      notification.warning({
        top: 92,
        message: error,
        duration: 5
      })
    }

    return error === ''
  }

  // 更新配置信息
  updateConfig = (config) => {
    this.setState({
      config: config
    })

    window.GLOB.customMenu = config
  }

  resetConfig = (config) => {
    this.setState({
      config: config,
      comloading: true
    }, () => {
      this.setState({
        comloading: false
      })
    })

    window.GLOB.customMenu = config
  }

  insert = (item) => {
    let config = fromJS(this.state.config).toJS()

    config.components.push(item)

    this.setState({config})
    window.GLOB.customMenu = config
  }

  setHomeView = () => {
    const { config } = this.state

    if (!is(fromJS(this.state.oriConfig || {}), fromJS(config || {}))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    }

    let param = {
      func: 's_kei_link_keyids_addupt',
      BID: sessionStorage.getItem('appId'),
      exec_type: 'x',
      LText: ''
    }

    let appViewList = sessionStorage.getItem('appViewList')
    appViewList = appViewList ? JSON.parse(appViewList) : []
    appViewList = appViewList.filter(item => item.keys_type !== 'index')

    appViewList.unshift({
      appkey: window.GLOB.appkey || '',
      bid: sessionStorage.getItem('appId') || '',
      kei_no: sessionStorage.getItem('kei_no') || '',
      keys_id: config.MenuID,
      keys_type: 'index',
      remark: config.MenuName
    })

    param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
    param.LText = param.LText.join(' union all ')
    param.LText = Utils.formatOptions(param.LText, 'x')

    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
    param.secretkey = Utils.encrypt('', param.timestamp)

    confirm({
      title: '确定设置本页面为首页吗？',
      content: '',
      onOk() {
        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
          }
        })
      },
      onCancel() {}
    })
  }

  setLoginView = () => {
    const { config } = this.state

    if (!is(fromJS(this.state.oriConfig || {}), fromJS(config || {}))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    }

    let param = {
      func: 's_kei_link_keyids_addupt',
      BID: sessionStorage.getItem('appId'),
      exec_type: 'x',
      LText: ''
    }

    let appViewList = sessionStorage.getItem('appViewList')
    appViewList = appViewList ? JSON.parse(appViewList) : []
    appViewList = appViewList.filter(item => item.keys_type !== 'login')

    appViewList.unshift({
      appkey: window.GLOB.appkey || '',
      bid: sessionStorage.getItem('appId') || '',
      kei_no: sessionStorage.getItem('kei_no') || '',
      keys_id: config.MenuID,
      keys_type: 'login',
      remark: config.MenuName
    })

    param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
    param.LText = param.LText.join(' union all ')
    param.LText = Utils.formatOptions(param.LText, 'x')

    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
    param.secretkey = Utils.encrypt('', param.timestamp)

    let hasLogin = config.components.findIndex(item => item.type === 'login') > -1

    confirm({
      title: '确定设置本页面为登录页吗？',
      content: hasLogin ? '' : '当前页面尚未添加登录组件。',
      onOk() {
        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
          }
        })
      },
      onCancel() {}
    })
  }

  checklog = () => {
    const { oriConfig, config } = this.state

    return is(fromJS(oriConfig), fromJS(config))
  }

  updateLogConfig = (config) => {
    config.open_edition = this.state.config.open_edition || ''
    
    this.setState({
      config: null
    }, () => {
      this.setState({
        config: config
      })
    })
    
    window.GLOB.customMenu = config
  }

  render () {
    const { view, loading, comloading, activeKey, settingshow, controlshow, MenuId, config, menuloading, eyeopen, needUpdate } = this.state

    return (
      <ConfigProvider locale={antdZhCN}>
        <DndProvider backend={HTML5Backend}>
          {view !== 'popview' ? <div className={'mk-pc-view '} id="mk-pc-design-view">
            {loading ? <Spin className="view-spin" size="large" /> : null}
            <div className={'menu-setting ' + (!settingshow ? 'hidden' : '')}>
              <div className="draw">
                {settingshow ? <DoubleLeftOutlined onClick={() => {sessionStorage.setItem('settingshow', 'false'); this.setState({settingshow: false})}}/> : null}
                {!settingshow ? <DoubleRightOutlined onClick={() => {sessionStorage.setItem('settingshow', 'true'); this.setState({settingshow: true})}}/> : null}
              </div>
              <div className="pc-setting-tools">
                <Collapse accordion activeKey={activeKey} bordered={false} onChange={(key) => this.setState({activeKey: key})}>
                  {/* 基本信息 */}
                  <Panel header="基本信息" key="basedata">
                    {config ? <>
                      <MenuForm
                        config={config}
                        MenuId={MenuId}
                        updateConfig={this.updateConfig}
                      />
                      <UrlFieldComponent config={config} updateConfig={this.updateConfig}/>
                      {/* 表名添加 */}
                      <TableComponent config={config} updatetable={this.updateConfig}/>
                      <Paragraph style={{padding: '15px 0px 0px 18px'}} copyable={{ text: MenuId }}>菜单ID</Paragraph>
                      <NormalCss config={config} updateConfig={this.updateConfig}/>
                    </> : null}
                  </Panel>
                  {/* 组件添加 */}
                  <Panel header="组件" key="component">
                    <SourceWrap />
                  </Panel>
                  <Panel header="元素" key="element">
                    <Modulecell />
                  </Panel>
                  <Panel header="页面样式" key="background">
                    {config ? <BgController config={config} updateConfig={this.updateConfig} /> : null}
                  </Panel>
                </Collapse>
              </div>
            </div>
            <div className={'menu-control ' + (!controlshow ? 'hidden' : '')}>
              <div className="draw">
                {controlshow ? <DoubleRightOutlined onClick={() => {sessionStorage.setItem('controlshow', 'false'); this.setState({controlshow: false})}}/> : null}
                {!controlshow ? <DoubleLeftOutlined onClick={() => {sessionStorage.setItem('controlshow', 'true'); this.setState({controlshow: true})}}/> : null}
              </div>
              <div className="wrap">
                <Button type="primary" className={needUpdate ? 'update-tip' : ''} id="save-config" onClick={this.submitConfig} loading={menuloading}>保存</Button>
                {config ? <Switch className="big" checkedChildren="启" unCheckedChildren="停" checked={config.enabled} onChange={this.onEnabledChange} /> : null}
                <ArrowLeftOutlined title="后退" className="back-view" onClick={this.backView}/>
                {config ? <Debug config={config}/> : null}
                <Button className="mk-border-purple" onClick={() => this.setState({eyeopen: !eyeopen})}>{!eyeopen ? <EyeOutlined /> : <EyeInvisibleOutlined />} 组件名</Button>
                <CreateView resetmenu={this.getAppMenus} />
                <PasteController insert={this.insert} />
                <TableNodes config={config} />
                <ViewNodes config={config} MenuId={MenuId}/>
                <SysInterface config={config} updateConfig={this.updateConfig}/>
                <PictureController/>
                <Quotecomponent config={config} updateConfig={this.updateConfig}/>
                <StyleCombControlButton menu={config} />
                <Button className="mk-border-green" onClick={this.setHomeView}><HomeOutlined /> 设为首页</Button>
                <Button className="mk-border-purple" onClick={this.setLoginView}><LoginOutlined/> 设为登录页</Button>
                <ReplaceField type="custom" config={config} updateConfig={this.resetConfig}/>
                <Transfer MenuID={MenuId} />
                {config ? <Versions MenuId={MenuId} Template="webPage" checklog={this.checklog} updateConfig={this.updateLogConfig}/> : null}
                <Button type="default" onClick={this.closeView}>关闭</Button>
              </div>
            </div>
            <div className={'menu-body menu-view' + (menuloading ? ' saving' : '') + (eyeopen ? ' eye-open' : '')}>
              {config && !comloading ? <MenuShell menu={config} handleList={this.updateConfig} /> : null}
            </div>
          </div> : <>
            <Header/>
            <PopView btn={this.state.popConfig} save={this.submitPopConfig} cancel={this.closePop}/>
          </>}
        </DndProvider>
        <StyleController />
        <StyleCombController />
        <ModalController />
      </ConfigProvider>
    )
  }
}

export default withRouter(MenuDesign)